@@ -139,8 +139,8 @@ class SignalHandler(object):
         self._ignore_signals = ('SIG_DFL', 'SIG_IGN')
         self._signals_by_name = dict((name, getattr(signal, name))
                                      for name in dir(signal)
-                                     if name.startswith("SIG")
-                                     and name not in self._ignore_signals)
+                                     if name.startswith("SIG") and
+                                     name not in self._ignore_signals)
         self.signals_to_name = dict(
             (sigval, name)
             for (name, sigval) in self._signals_by_name.items())
@@ -203,7 +203,9 @@ class SignalHandler(object):
         For Python 3.5 and later, deal with the changes in PEP 475 that prevent
         a signal from interrupting eventlet's call to poll() or sleep().
         """
-        self.__force_interrupt_on_signal = sys.version_info >= (3, 5)
+        select_module = eventlet.patcher.original('select')
+        self.__force_interrupt_on_signal = (sys.version_info >= (3, 5) and
+                                            hasattr(select_module, 'poll'))
 
         if self.__force_interrupt_on_signal:
             try:
