@@ -227,7 +227,7 @@ class ServiceLauncherTest(ServiceTestBase):
 
         os.kill(start_workers[0], signal.SIGHUP)
         # Wait at most 5 seconds to respawn a worker
-        cond = lambda: start_workers == self._get_workers()
+        cond = lambda: start_workers != self._get_workers()
         timeout = 5
         self._wait(cond, timeout)
 
@@ -477,6 +477,31 @@ class ProcessLauncherTest(base.ServiceBaseTestCase):
             m.assert_called_once_with(signal.SIGTERM, 'test')
         signal_handler.clear()
 
+    @mock.patch('sys.version_info', (3, 5))
+    def test_setup_signal_interruption_no_select_poll(self):
+        # NOTE(claudiub): SignalHandler is a singleton, which means that it
+        # might already be initialized. We need to clear to clear the cache
+        # in order to prevent race conditions between tests.
+        service.SignalHandler.__class__._instances.clear()
+        with mock.patch('eventlet.patcher.original',
+                        return_value=object()) as get_original:
+            signal_handler = service.SignalHandler()
+            get_original.assert_called_with('select')
+        self.addCleanup(service.SignalHandler.__class__._instances.clear)
+        self.assertFalse(
+            signal_handler._SignalHandler__force_interrupt_on_signal)
+
+    @mock.patch('sys.version_info', (3, 5))
+    def test_setup_signal_interruption_select_poll(self):
+        # NOTE(claudiub): SignalHandler is a singleton, which means that it
+        # might already be initialized. We need to clear to clear the cache
+        # in order to prevent race conditions between tests.
+        service.SignalHandler.__class__._instances.clear()
+        signal_handler = service.SignalHandler()
+        self.addCleanup(service.SignalHandler.__class__._instances.clear)
+        self.assertTrue(
+            signal_handler._SignalHandler__force_interrupt_on_signal)
+
     @mock.patch('signal.alarm')
     @mock.patch("os.kill")
     @mock.patch("oslo_service.service.ProcessLauncher.stop")
@@ -613,13 +638,19 @@ class EventletServerProcessLauncherTest(base.ServiceBaseTestCase):
 
     def run_server(self):
         queue = multiprocessing.Queue()
+        # NOTE(bnemec): process_time of 5 needs to be longer than the graceful
+        # shutdown timeout in the "exceeded" test below, but also needs to be
+        # shorter than the timeout in the regular graceful shutdown test.
         proc = multiprocessing.Process(target=eventlet_service.run,
                                        args=(queue,),
-                                       kwargs={'workers': self.workers})
+                                       kwargs={'workers': self.workers,
+                                               'process_time': 5})
         proc.start()
 
         port = queue.get()
         conn = socket.create_connection(('127.0.0.1', port))
+        # Send request to make the connection active.
+        conn.sendall(b'GET / HTTP/1.1\r\nHost: localhost\r\n\r\n')
 
         # NOTE(blk-u): The sleep shouldn't be necessary. There must be a bug in
         # the server implementation where it takes some time to set up the
@@ -648,10 +679,14 @@ class EventletServerProcessLauncherTest(base.ServiceBaseTestCase):
         # connected.
         os.kill(proc.pid, signal.SIGTERM)
 
-        # server with graceful shutdown must wait forewer if
+        # server with graceful shutdown must wait forever if
         # option graceful_shutdown_timeout is not specified.
-        # we can not wait forever ... so 3 seconds are enough
-        time.sleep(3)
+        # we can not wait forever ... so 1 second is enough.
+        # NOTE(bnemec): In newer versions of eventlet that drop idle
+        # connections, this needs to be long enough to allow the signal
+        # handler to fire but short enough that our request doesn't complete
+        # or the connection will be closed and the server will stop.
+        time.sleep(1)
 
         self.assertTrue(proc.is_alive())
 
